
module DDR2_Control_4Port
(
	//	HOST Side
	REF_CLK,
	RESET_N,
		  
	GLOBAL_RES,
	
	//	FIFO Write Side 1
	WR1_DATA,
	WR1,
	WR1_ADDR,
	WR1_MAX_ADDR,
	WR1_LENGTH,
	WR1_LOAD,
	WR1_CLK,
	WR1_FULL,
	WR1_USE,
	
	//	FIFO Write Side 2
	WR2_DATA,
	WR2,
	WR2_ADDR,
	WR2_MAX_ADDR,
	WR2_LENGTH,
	WR2_LOAD,
	WR2_CLK,
	WR2_FULL,
	WR2_USE,

	//	FIFO Read Side 1
	RD1_DATA,
	RD1,
	RD1_ADDR,
	RD1_MAX_ADDR,
	RD1_LENGTH,
	RD1_LOAD,	
	RD1_CLK,

	//	FIFO Read Side 2
	RD2_DATA,
	RD2,
	RD2_ADDR,
	RD2_MAX_ADDR,
	RD2_LENGTH,
	RD2_LOAD,
	RD2_CLK,

	//	DDR Side
	mem_odt,
	mem_cs_n,
	mem_cke,
	mem_addr,
	mem_ba,
	mem_ras_n,
	mem_cas_n,
	mem_we_n,
	mem_dm,
	mem_clk,
	mem_clk_n,
	mem_dq,
	mem_dqs,
	
	local_rdata
	
);

//`define  ASIZE           	24      	// total address width of the DDR2
//`define	DSIEZE_IN			16			//	data width of the CMOS
//`define	DSIEZE_DDR2			32			//	data width of the DDR2 IP 
//`define	RATIO					2			// the data width ratio of `DSIEZE_DDR2/`DSIEZE_IN

`define  ASIZE           	25      	// total address width of the DDR2
`define	DSIEZE_IN			16			//	data width of the CMOS
`define	DSIEZE_DDR2			64			//	data width of the DDR2 IP 
`define	RATIO					4			// the data width ratio of `DSIEZE_DDR2/`DSIEZE_IN

//	HOST Side
input                           	REF_CLK;                	//System Clock
input                           	RESET_N;                	//System Reset

output									GLOBAL_RES;
//	FIFO Write Side 1
input [`DSIEZE_IN-1:0]         	WR1_DATA;               	//Data input
input										WR1;								//Write Request
input	[`ASIZE-1:0]					WR1_ADDR;						//Write start address
input	[`ASIZE-1:0]					WR1_MAX_ADDR;					//Write max address
input	[`ASIZE-1:0]					WR1_LENGTH;						//Write length
input										WR1_LOAD;				      //Write register load & fifo clear
input										WR1_CLK;				         //Write fifo clock
output									WR1_FULL;						//Write fifo full
output[`ASIZE-1:0]					WR1_USE;							//Write fifo usedw
//	FIFO Write Side 2
input [`DSIEZE_IN-1:0]         	WR2_DATA;               	//Data input
input										WR2;								//Write Request
input	[`ASIZE-1:0]					WR2_ADDR;						//Write start address
input	[`ASIZE-1:0]					WR2_MAX_ADDR;					//Write max address
input	[`ASIZE-1:0]					WR2_LENGTH;						//Write length
input										WR2_LOAD;						//Write register load & fifo clear
input										WR2_CLK;							//Write fifo clock
output									WR2_FULL;						//Write fifo full
output[`ASIZE-1:0]					WR2_USE;							//Write fifo usedw
//	FIFO Read Side 1
output[`DSIEZE_IN-1:0]        	RD1_DATA;               	//Data output
input										RD1;								//Read Request
input	[`ASIZE-1:0]					RD1_ADDR;						//Read start address
input	[`ASIZE-1:0]					RD1_MAX_ADDR;					//Read max address
input	[`ASIZE-1:0]					RD1_LENGTH;						//Read length
input										RD1_LOAD;						//Read register load & fifo clear
input										RD1_CLK;							//Read fifo clock

//	FIFO Read Side 2
output[`DSIEZE_IN-1:0]         	RD2_DATA;               	//Data output			DSIZE=32;    		(Image_Read_Data),
input							  			RD2;								//Read Request							 		(Image_Read),	
input	[`ASIZE-1:0]					RD2_ADDR;						//Read start address				    		(0),		
input	[`ASIZE-1:0]					RD2_MAX_ADDR;				   //Read max address							(640*480)
input	[`ASIZE-1:0]					RD2_LENGTH;						//Read length								   (9'h100=256)
input										RD2_LOAD;						//Read register load & fifo clear   	(!DLY_RST_0),
input										RD2_CLK;							//Read fifo clock							   (Image_Read_CLK),

output[0:0]								mem_odt;
output[0:0]								mem_cs_n;
output[0:0]								mem_cke;
output[12:0]							mem_addr;
output[2:0]								mem_ba;
output									mem_ras_n;
output									mem_cas_n;
output									mem_we_n;
//output[1:0]								mem_dm;
output[1:0]								mem_dm;
inout	[0:0]								mem_clk;
inout	[0:0]								mem_clk_n;
//inout	[15:0]							mem_dq;
//inout	[1:0]								mem_dqs;
inout	[31:0]							mem_dq;
inout	[3:0]								mem_dqs;

//	Internal Registers/Wires
//	Controller
reg	[`ASIZE-1:0]					mADDR;							//Internal address
reg	[`ASIZE-1:0]					mLENGTH;							//Internal length
reg	[`ASIZE-1:0]					rWR1_ADDR;						//Register write address				
reg	[`ASIZE-1:0]					rWR1_MAX_ADDR;					//Register max write address				
reg	[`ASIZE-1:0]					rWR1_LENGTH;					//Register write length
reg	[`ASIZE-1:0]					rWR2_ADDR;						//Register write address				
reg	[`ASIZE-1:0]					rWR2_MAX_ADDR;					//Register max write address				
reg	[`ASIZE-1:0]					rWR2_LENGTH;					//Register write length
reg	[`ASIZE-1:0]					rRD1_ADDR;						//Register read address
reg	[`ASIZE-1:0]					rRD1_MAX_ADDR;					//Register max read address
reg	[`ASIZE-1:0]					rRD1_LENGTH;					//Register read length
reg	[`ASIZE-1:0]					rRD2_ADDR;						//Register read address
reg	[`ASIZE-1:0]					rRD2_MAX_ADDR;					//Register max read address
reg	[`ASIZE-1:0]					rRD2_LENGTH;					//Register read length
reg	[1:0]								WR_MASK;							//Write port active mask
reg	[1:0]								RD_MASK;							//Read port active mask
reg										mWR_DONE;						//Flag write done, 1 pulse SDR_CLK
reg										mRD_DONE;						//Flag read done, 1 pulse SDR_CLK
reg										mWR;								//Internal WR edge capture
reg										mRD;								//Internal RD edge capture

wire  [`DSIEZE_DDR2-1:0]     	 	WR_DATA1;                	//Controller Data input 1
wire  [`DSIEZE_DDR2-1:0]      	WR_DATA2;                	//Controller Data input 2

//	FIFO Control
wire	[`ASIZE-1:0]					write_side_fifo_rusedw1;
wire	[`ASIZE-1:0]					read_side_fifo_wusedw1;
wire	[`ASIZE-1:0]					write_side_fifo_rusedw2;
wire	[`ASIZE-1:0]					read_side_fifo_wusedw2;
//	DRAM Internal Control

wire                            	CLK;

wire	[3:0]								local_size;	
wire										local_ready;
//wire	[3:0]								local_be;
wire	[7:0]								local_be;
reg	[`DSIEZE_DDR2-1:0]			local_wdata;
output wire	[`DSIEZE_DDR2-1:0]			local_rdata;
reg	[`ASIZE:0]						local_address;
reg										local_read_req;
reg										local_write_req;
reg										local_burstbegin;
wire										local_init_done;
wire										local_rdata_valid;

//assign	local_be		= 4'hF;
assign	local_be		= 8'hFF;
assign	local_size 	= 4'h8;
assign	GLOBAL_RES 	= local_init_done;

DDR2		I_DDR2
(
	.pll_ref_clk			(REF_CLK),
	.phy_clk					(CLK),
	
	.soft_reset_n			(1'b1),
	.global_reset_n		(RESET_N),

	.local_address			(local_address),	
	.local_write_req		(local_write_req),
	.local_wdata			(local_wdata),	
	.local_read_req		(local_read_req),
	.local_rdata			(local_rdata),	
	.local_burstbegin		(local_burstbegin),

	.local_be				(local_be),
	.local_size				(local_size),
	.local_ready			(local_ready),
	.local_rdata_valid	(local_rdata_valid),
	.local_init_done		(local_init_done),

	.mem_odt					(mem_odt),
	.mem_cs_n				(mem_cs_n),
	.mem_cke					(mem_cke),
	.mem_addr				(mem_addr),
	.mem_ba					(mem_ba),
	.mem_ras_n				(mem_ras_n),
	.mem_cas_n				(mem_cas_n),
	.mem_we_n				(mem_we_n),
	.mem_dm					(mem_dm),
	.mem_clk					(mem_clk),
	.mem_clk_n				(mem_clk_n),
	.mem_dq					(mem_dq),
	.mem_dqs					(mem_dqs)
);

reg		WR_FIFO_EN;

WR_FIFO 	WR_FIFO_1
(
	.data						(WR1_DATA),
	.wrreq					(WR1),
	.wrclk					(WR1_CLK),
	.aclr						((!local_init_done) || WR1_LOAD),
	.rdreq					(WR_FIFO_EN & WR_MASK[0]),
	.rdclk					(CLK),
	.q							(WR_DATA1),
	.wrfull					(WR1_FULL),
	.wrusedw					(WR1_USE),
	.rdusedw					(write_side_fifo_rusedw1)
);

WR_FIFO 	WR_FIFO_2
(
	.data						(WR2_DATA),
	.wrreq					(WR2),
	.wrclk					(WR2_CLK),
	.aclr						((!local_init_done) || WR2_LOAD),
	.rdreq					(WR_FIFO_EN & WR_MASK[1]),
	.rdclk					(CLK),
	.q							(WR_DATA2),
	.wrfull					(WR2_FULL),
	.wrusedw					(WR2_USE),
	.rdusedw					(write_side_fifo_rusedw2)
);

RD_FIFO 	RD_FIFO_1
(
	.data						(local_rdata),
	.wrreq					(local_rdata_valid & RD_MASK[0]),
	.wrclk					(CLK),
	.aclr						((!local_init_done) || RD1_LOAD),
	.rdreq					(RD1),
	.rdclk					(RD1_CLK),
	.q							(RD1_DATA),
	.wrusedw					(read_side_fifo_wusedw1)
);
				
RD_FIFO 	RD_FIFO_2
(
	.data						(local_rdata),
	.wrreq					(local_rdata_valid & RD_MASK[1]),
	.wrclk					(CLK),
	.aclr						((!local_init_done) || RD2_LOAD),
	.rdreq					(RD2),
	.rdclk					(RD2_CLK),
	.q							(RD2_DATA),
	.wrusedw					(read_side_fifo_wusedw2)
);

reg	[9:0]		Write_NUM;
reg	[9:0]		Read_NUM;			

reg	[3:0]		WR_Step;
reg	[3:0]		RD_Step;

reg	[7:0]		WR_Time;
reg	[7:0]		RD_Time;
	
always@(posedge CLK or negedge local_init_done)
begin
	if(!local_init_done)
	begin
		mWR_DONE				<=	0;
		mRD_DONE				<=	0;
		local_write_req	<= 0;
		local_read_req 	<= 0;	
		local_burstbegin 	<= 0;
		local_address		<= 0;
		local_wdata			<= 0;
		Write_NUM 			<= 0;
		Read_NUM	 			<= 0;
		WR_FIFO_EN			<= 0;
		
		WR_Step				<= 0;
		RD_Step				<= 0;	
	end
	else
	begin
		case(WR_MASK)
			2'b01	:
						local_wdata <= WR_DATA1;
			2'b10	:
						local_wdata <= WR_DATA2;
			default:
					local_wdata <= 0;
		endcase
	
		if(mWR)
		begin
			if(local_ready)
			begin
				case(WR_Step)
					4'd0: 
						begin
							local_write_req <= 0; local_burstbegin <= 0; Write_NUM <= 0; WR_Time <= 0; WR_FIFO_EN <= 0; WR_Step  <= 4'd1;	
						end
					4'd1:
						begin
							WR_FIFO_EN <= 1; WR_Step  <= 4'd2;	
						end										
					4'd2: 	
						begin
							local_write_req <= 1; local_burstbegin <= 1; local_address<= mADDR + WR_Time * local_size; WR_FIFO_EN <= 1;
								
							WR_Step <= 4'd3;
						end
					4'd3:
						begin	
							local_burstbegin <= 1'b0;
							
							if(Write_NUM >= local_size - 2)
								WR_FIFO_EN <= 0;
							else
								WR_FIFO_EN <= 1;
							
							if(Write_NUM < local_size - 1)
							begin
								Write_NUM <= Write_NUM + 1'b1; WR_Step <= 4'd3;								
							end
							else
							begin
								Write_NUM <= 0; local_write_req <= 0;
								
								if(WR_Time < mLENGTH/local_size -1)
								begin
									WR_Time <= WR_Time + 8'd1; WR_Step <= 4'd1; 
								end
								else
								begin
									mWR_DONE <= 1; WR_Step <= 4'd0; 
								end			
							end	
						end		
					default:
						WR_Step <= 4'd0;	
				endcase	
			end
			else
			begin
				local_write_req <= 0; WR_FIFO_EN <= 0;
			end
		end
		else	
		begin
			mWR_DONE <= 0; local_write_req <= 0; local_burstbegin <= 0; Write_NUM <= 0; WR_Time <= 0; WR_FIFO_EN <= 0; WR_Step <= 4'd0; 
		end

		if(mRD)
		begin
			case(RD_Step)
				4'd0: 
					begin
						local_burstbegin <= 0; local_read_req <= 0; Read_NUM	<= 0; RD_Time <= 0; RD_Step <= 4'd1;						
					end
				4'd1: 	
					begin
						local_read_req <= 1; local_burstbegin <= 1'b1; Read_NUM <= 0; local_address	<= mADDR + RD_Time * local_size ; RD_Step <= 4'd2;	
					end	
				4'd2:
					begin	
						if(local_ready)
						begin
							local_burstbegin <= 1'b0; local_read_req <= 0;	
								
							if(local_rdata_valid)
							begin
									if(Read_NUM < local_size -1)
									begin
										Read_NUM <= Read_NUM + 1'b1; RD_Step <= 4'd2;	
									end	
									else
									begin
										Read_NUM <= 0;
										
										if(RD_Time < mLENGTH/local_size -1)
										begin
											RD_Time <= RD_Time + 8'd1; RD_Step <= 4'd1;
										end	
										else
										begin
											RD_Time	<= 0; mRD_DONE <= 1; RD_Step <= 4'd0;	
										end															
									end
							end		
						end
						else
							RD_Step <= 4'd2;	
					end
				default:
					RD_Step <= 4'd0;	
			endcase				
		end
		else
		begin
			mRD_DONE <= 0; local_burstbegin <= 0; local_read_req <= 0; Read_NUM	<= 0; RD_Time <= 0; RD_Step <= 4'd0;	
		end
	end
end
	
//	Internal Address & Length Control
always@(posedge CLK or negedge local_init_done)
begin
	if(!local_init_done)
	begin
		rWR1_ADDR		<=	WR1_ADDR/`RATIO;
		rWR1_MAX_ADDR	<=	WR1_MAX_ADDR;
		rWR2_ADDR		<=	WR2_ADDR/`RATIO;
		rWR2_MAX_ADDR	<=	WR2_MAX_ADDR;
		rRD1_ADDR		<=	RD1_ADDR/`RATIO;
		rRD1_MAX_ADDR	<=	RD1_MAX_ADDR;
		rRD2_ADDR		<=	RD2_ADDR/`RATIO;
		rRD2_MAX_ADDR	<=	RD2_MAX_ADDR;
		rWR1_LENGTH		<=	WR1_LENGTH;
		rRD1_LENGTH		<=	RD1_LENGTH;
		rWR2_LENGTH		<=	WR2_LENGTH;
		rRD2_LENGTH		<=	RD2_LENGTH;
	end
	else
	begin
		//	Write Side 1
		if(WR1_LOAD)
		begin
			rWR1_ADDR	<=	WR1_ADDR/`RATIO;
			rWR1_LENGTH	<=	WR1_LENGTH;
		end
		else if(mWR_DONE&WR_MASK[0])
		begin
			if(rWR1_ADDR<rWR1_MAX_ADDR/`RATIO-rWR1_LENGTH)
			rWR1_ADDR	<=	rWR1_ADDR + rWR1_LENGTH;
			else
			rWR1_ADDR	<=	WR1_ADDR/`RATIO;
		end
		//	Write Side 2
		if(WR2_LOAD)
		begin
			rWR2_ADDR	<=	WR2_ADDR/`RATIO;
			rWR2_LENGTH	<=	WR2_LENGTH;
		end
		else if(mWR_DONE&WR_MASK[1])
		begin
			if(rWR2_ADDR<rWR2_MAX_ADDR/`RATIO-rWR2_LENGTH)
			rWR2_ADDR	<=	rWR2_ADDR+rWR2_LENGTH;
			else
			rWR2_ADDR	<=	WR2_ADDR/`RATIO;
		end
		//	Read Side 1
		if(RD1_LOAD)
		begin
			rRD1_ADDR	<=	RD1_ADDR/`RATIO;
			rRD1_LENGTH	<=	RD1_LENGTH;
		end
		else if(mRD_DONE&RD_MASK[0])
		begin
			if(rRD1_ADDR<rRD1_MAX_ADDR/`RATIO-rRD1_LENGTH)
			rRD1_ADDR	<=	rRD1_ADDR+rRD1_LENGTH;
			else
			rRD1_ADDR	<=	RD1_ADDR/`RATIO;
		end
		//	Read Side 2
		if(RD2_LOAD)
		begin
			rRD2_ADDR	<=	RD2_ADDR/`RATIO;
			rRD2_LENGTH	<=	RD2_LENGTH;
		end
		else if(mRD_DONE&RD_MASK[1])
		begin
			if(rRD2_ADDR<rRD2_MAX_ADDR/`RATIO-rRD2_LENGTH)
			rRD2_ADDR	<=	rRD2_ADDR+rRD2_LENGTH;
			else
			rRD2_ADDR	<=	RD2_ADDR/`RATIO;
		end
	end
end

//****************Global controller**************************//
/*
	

*/
always@(posedge CLK or negedge local_init_done)
begin
	if(!local_init_done)
	begin
		mWR		<=	0;
		mRD		<=	0;
		mADDR		<=	0;
		mLENGTH	<=	0;
		WR_MASK	<=	2'b00;
		RD_MASK	<=	2'b00;
	end
	else
	begin
		if( (mWR==0) && (mRD==0) &&
			(WR_MASK==2'b0)	&&	(RD_MASK==2'b0) &&
			(WR1_LOAD==1'b0)	&&	(RD1_LOAD==1'b0) &&
			(WR2_LOAD==1'b0)	&&	(RD2_LOAD==1'b0) )
		begin
		
		//	Write Side 1
			 if( (write_side_fifo_rusedw1 >= rWR1_LENGTH) && (rWR1_LENGTH!=0) )
			begin
				mADDR		<=	rWR1_ADDR;
				mLENGTH	<=	rWR1_LENGTH;
				WR_MASK	<=	2'b01;
				RD_MASK	<=	2'b00;
				mWR		<=	1;
				mRD		<=	0;
			end
			//	Write Side 2
			else if( (write_side_fifo_rusedw2 >= rWR2_LENGTH) && (rWR2_LENGTH!=0) )
			begin
				mADDR	<=	rWR2_ADDR;
				mLENGTH	<=	rWR2_LENGTH;
				WR_MASK	<=	2'b10;
				RD_MASK	<=	2'b00;
				mWR		<=	1;
				mRD		<=	0;
			end
		//	Read Side 1
			else if( (read_side_fifo_wusedw1 < rRD1_LENGTH) )
			begin
				mADDR	<=	rRD1_ADDR;
				mLENGTH	<=	rRD1_LENGTH;
				WR_MASK	<=	2'b00;
				RD_MASK	<=	2'b01;
				mWR		<=	0;
				mRD		<=	1;				
			end
			//	Read Side 2
			else if( (read_side_fifo_wusedw2 < rRD2_LENGTH) )
			begin
				mADDR	<=	rRD2_ADDR;
				mLENGTH	<=	rRD2_LENGTH;
				WR_MASK	<=	2'b00;
				RD_MASK	<=	2'b10;
				mWR		<=	0;
				mRD		<=	1;
			end	
		end
		if(mWR_DONE)
		begin
			WR_MASK	<=	0;
			mWR		<=	0;
		end
		if(mRD_DONE)
		begin
			RD_MASK	<=	0;
			mRD		<=	0;
		end
	end
end

endmodule
