module ov5640_ddr2_vga(
	Clk50M,
	Clk27M,
	Rst_n,

	Camera_sclk,
	Camera_sdat,
	Camera_vsync,
	Camera_href,
	Camera_pclk,
	Camera_xclk,
	Camera_data,
	Camera_rst_n,
	Camera_pwdn,
	
	DDR2_ADDR,
	DDR2_BA,
	DDR2_CAS,
	DDR2_CKE,
	DDR2_CLK,
	DDR2_CLK_N,
	DDR2_CS,
	DDR2_DM,
	DDR2_DQ,
	DDR2_DQS,
	DDR2_ODT,
	DDR2_RAS,
	DDR2_WE,
	
	vga_data,
	vga_hs,
	vga_vs,
	vga_clk,
	vga_de
);
	input               	Clk50M;     //系统时钟
	input               	Clk27M;     //系统时钟
	input               	Rst_n;      //系统复位信号
	
	//cmos interface
	output					Camera_sclk;
	inout						Camera_sdat;
	input						Camera_vsync;
	input						Camera_href;
	input						Camera_pclk;
	output					Camera_xclk;
	input 	[7:0]			Camera_data;
	output					Camera_rst_n;
	output					Camera_pwdn;

	//ddr control
	output	[12:0]		DDR2_ADDR;
	output	[2:0]			DDR2_BA;
	output					DDR2_CAS;
	output					DDR2_CKE;
	inout						DDR2_CLK;
	inout						DDR2_CLK_N;
	output					DDR2_CS;
	output	[3:0]			DDR2_DM;
	inout		[31:0]		DDR2_DQ;
	inout		[3:0]			DDR2_DQS;
	output					DDR2_ODT;
	output					DDR2_RAS;
	output					DDR2_WE;
	
	//vga output
	output  	[23:0] 		vga_data;
	output       			vga_hs;
	output       			vga_vs;
	output       			vga_clk;
	output       			vga_de;
	
	reg [19:0]cnt;
	reg sys_rst_n;
	always@(posedge Clk50M or negedge Rst_n)
	begin
		if(!Rst_n)
			cnt <= 20'd0;
		else if(cnt < 20'hffff0)
			cnt <= cnt + 8'd1;
		else
			cnt <= cnt;
	end	
	
	always@(posedge Clk50M or negedge Rst_n)
	begin
		if(!Rst_n)
			sys_rst_n <= 1'b0;
		else if(cnt >= 20'hffff0)
			sys_rst_n <= 1'b1;
		else
			sys_rst_n <= 1'b0;
	end
	
	wire GLOBAL_RES;
	wire clk_50m;
	wire clk_24m;
	wire clk_vga;
	assign Camera_xclk = clk_24m;
	pll pll(
		.inclk0(Clk27M),
		.c0(clk_50m),
		.c1(clk_24m),
		.c2(clk_vga)
	);
	
	localparam RGB = 0;
	localparam JPEG = 1;
	
	parameter IMAGE_WIDTH  = 1920; //图片宽度
	parameter IMAGE_HEIGHT = 1080; //图片高度
	parameter IMAGE_FLIP   = 0;   //0：不翻转，1：上下翻转
	parameter IMAGE_MIRROR = 0;   //0：不镜像，1：左右镜像

	//摄像头初始化配置
	wire Init_Done;
	camera_init
	#(
		.IMAGE_TYPE(RGB),
		.IMAGE_WIDTH(IMAGE_WIDTH),
		.IMAGE_HEIGHT(IMAGE_HEIGHT),
		.IMAGE_FLIP(IMAGE_FLIP),
		.IMAGE_MIRROR(IMAGE_MIRROR)
	)
	camera_init(
		.Clk(clk_50m),
		.Rst_n(sys_rst_n & GLOBAL_RES),
		.Init_Done(Init_Done),
		.camera_rst_n(Camera_rst_n),
		.camera_pwdn(Camera_pwdn),
		.i2c_sclk(Camera_sclk),
		.i2c_sdat(Camera_sdat)
	);
	
	//摄像头1,摄像头2图像输出
	wire fifo_aclr;
	wire fifo_wrreq;
	wire [15:0] fifo_wrdata;
	DVP_Capture DVP_Capture(
		.Rst_n(Init_Done),
		.PCLK(Camera_pclk),
		.Vsync(Camera_vsync),
		.Href(Camera_href),
		.Data(Camera_data),
		.ImageState(fifo_aclr),
		.DataValid(fifo_wrreq),
		.DataPixel(fifo_wrdata),
		.Xaddr(),
		.Yaddr()
	);
	
	wire [15:0] RGB_DATA;
	wire DataReq;
	DDR2_Control_4Port I_DDR2_Control_4Port
	(
		//	HOST Side
		.REF_CLK			(Clk50M),
		.RESET_N			(Rst_n & sys_rst_n),
			  
		.GLOBAL_RES		(GLOBAL_RES),
		
		//	FIFO Write Side 1
		.WR1_DATA		(fifo_wrdata),
		.WR1				(fifo_wrreq),
		.WR1_ADDR		(0),
		.WR1_MAX_ADDR	(IMAGE_WIDTH*IMAGE_HEIGHT),
		.WR1_LENGTH		(256),
		.WR1_LOAD		(!GLOBAL_RES),
		.WR1_CLK			(Camera_pclk),
		
		//	FIFO Read Side 1
		.RD1_DATA		(RGB_DATA),
		.RD1				(DataReq),
		.RD1_ADDR		(0),
		.RD1_MAX_ADDR	(IMAGE_WIDTH*IMAGE_HEIGHT),
		.RD1_LENGTH		(256),
		.RD1_LOAD		(!GLOBAL_RES | frame_begin),	
		.RD1_CLK			(clk_vga),

		//	DDR Side
		.mem_odt			(DDR2_ODT),
		.mem_cs_n		(DDR2_CS),
		.mem_cke			(DDR2_CKE),
		.mem_addr		(DDR2_ADDR),
		.mem_ba			(DDR2_BA),
		.mem_ras_n		(DDR2_RAS),
		.mem_cas_n		(DDR2_CAS),
		.mem_we_n		(DDR2_WE),
		.mem_dm			(DDR2_DM),
		.mem_clk			(DDR2_CLK),
		.mem_clk_n		(DDR2_CLK_N),
		.mem_dq			(DDR2_DQ),
		.mem_dqs			(DDR2_DQS)
	);
	
	wire [4:0]  video_r;
	wire [5:0]  video_g;
	wire [4:0]  video_b;
	wire frame_begin;

	disp_driver disp_driver(
		.ClkDisp(clk_vga),
		.Rst_n(Rst_n),
		.Data(RGB_DATA),
		.DataReq(DataReq),
		.H_Addr(),
		.V_Addr(),
		.Disp_HS(vga_hs),
		.Disp_VS(vga_vs),
		.Disp_Red(video_r),
		.Disp_Green(video_g),
		.Disp_Blue(video_b),
		.frame_begin(frame_begin),
		.Disp_DE(vga_de),
		.Disp_PCLK(vga_clk)
	);
	
	//vga display
	assign vga_data = {video_r, 3'd0, video_g, 2'd0, video_b, 3'd0}; //vga数据输出

endmodule
